# debugging prelude
import sys, pdb, traceback
sys.excepthook = lambda *x: (traceback.print_exception(*x), pdb.pm())
# end debugging prelude

from peak.api import *
from citykid import data
import os


class bunch(object):
    def __init__(self,items):
        self.__dict__ = dict(items)

    def __repr__(self):
        return "bunch(%r)" % (self.__dict__.keys(),)

























class CityApp(commands.AbstractCommand):

    fileURL = binding.Make(
        lambda self: "ref:ado@dsn:Provider=Microsoft.Jet.OLEDB.4.0;"
            "Data Source="+os.path.realpath(self.parsed_args[0])
    )

    db = binding.Obtain(naming.Indirect('fileURL'))

    items = binding.Make(data.ItemDM)
    templateDM = binding.Make(data.TemplateDM)

    language = 'English'
    family   = 'Kid'

    ixLanguage = binding.Make(
        lambda self: (
            ~self.db(
                "select ixLanguage from tblLanguage"
                " where sLanguage=? and fDeleted=0",
                (self.language,)
            )
        )[0]
    )

    ixFamily = binding.Make(
        lambda self: (
            ~self.db(
                "select ixTemplateFamily from tblTemplateFamily"
                " where sName=? and fDeleted=0",
                (self.family,)
            )
        )[0]
    )

    vars = binding.Make(
        lambda self: bunch(self.db("select sName, sValue from tblVariables"))
    )



    templates = binding.Make(
        lambda self: bunch(
            [(k,self.templateDM[v])
                for k,v in self.db(
                    "select sName,ixTemplateStructure"
                    " from tblTemplateStructure where fDeleted=0"
                )
            ]
        )
    )

    [dispatch.as_(binding.Make)]
    def all(self):
        items = [
            (item.filed,item)
                for parent,children in self.hierarchy.items()
                    for item in children
                        if isinstance(item,data.Article)
        ]
        items.sort(); items.reverse()   # descending by filed date
        return [item[1] for item in items]

    [dispatch.as_(binding.Make)]
    def hierarchy(self):
        d={}
        for item in self.items:
            if item.active: d.setdefault(item.parent,[]).append(item)
        return d

    staging = binding.Make(
        lambda self: os.path.realpath(self.parsed_args[0]) + '.staging-area'
    )

    def walk(self, parent=None):
        stack = [parent]
        while stack:
            item = stack.pop()
            if item: yield item
            stack.extend(self.hierarchy.get(item,()))


    [dispatch.as_(binding.Make)]
    def byAudience(self):
        d = dict([(name,[]) for name in self.items.audiences.values() if name])
        for article in self.all:
            if article.audience=='(Everyone)':
                for name in d: d[name].append(article)
            else:
                d[article.audience].append(article)
        return d

    [dispatch.as_(binding.Make)]
    def byKeyword(self):
        d = {}
        for article in self.all:
            for key in article.keywords.split():
                d.setdefault(key,[]).append(article)
        return d

    [dispatch.as_(binding.Make)]
    def byName(self):
        d = {}
        for article in self.all:
            d.setdefault(article.publishAs,[]).append(article)
        return d

    byPath = binding.Make(
        lambda self: dict([(article.path,article) for article in self.all])
    )

    [dispatch.as_(binding.Make)]
    def byTemplate(self):
        toName = dict([(v,k) for k,v in self.templates.__dict__.items()])
        d = {}
        for article in self.all:
            d.setdefault(toName[article.template],[]).append(article)
        return d





    params = binding.Make(
        lambda self: dict(
            templates = self.templates,
            site      = self.vars,
            all       = self.all,
            cmd       = self
        )
    )

    def load_template(self, text):
        import kid
        return kid.load_template(text, cache=False, ns=self.params)

    def call_template(self, module, **kw):
        kw.update(self.params)
        return module.Template(**kw)

























class CityKid(CityApp):

    usage = """Usage: citykid filename.cty [itemnumber]
"""

    dispatch.as_(binding.Make)
    def itemNo(self):
        if len(self.parsed_args)>1:
            try:
                return int(self.parsed_args[1])
            except ValueError:
                raise commands.InvocationError(
                    "Item # must be an integer", self.parsed_args[1]
                )

    def _checkArgs(self):
        args = self.parsed_args
        if len(args)<1 or len(args)>2:
            raise commands.InvocationError(
                "Must supply a CityDesk filename and optional item #"
            )

    def _run(self):
        self._checkArgs()
        storage.begin(self)
        start = self.items.get(self.itemNo)
        for item in self.walk(start):
            print "item path = ", item.filename(self.staging)
            item.save(self.staging)
            if isinstance(item, data.Folder):
                item.delete_orphans(self.staging, self.hierarchy.get(item,[]))
        if start:
            if isinstance(start, data.Folder) or start.path.endswith('.html'):
                os.startfile(start.filename(self.staging))
        else:
            os.startfile(self.staging)
        storage.abort(self)




class CityDump(CityKid):

    usage = """Usage: citykid filename.cty [itemnumber]
"""
    staging = binding.Make(
        lambda self: os.path.realpath(self.parsed_args[0]) + '.dump'
    )

    def _run(self):
        self._checkArgs()
        storage.begin(self)
        start = self.items.get(self.itemNo)
        for item in self.walk(start):
            print "item path = ", item.filename(self.staging)
            item.dump(self.staging)
            if isinstance(item, data.Folder):
                item.delete_orphans(
                    self.staging, self.hierarchy.get(item,[]), dump=True
                )
        if start:
            if isinstance(start, data.Folder) or start.path.endswith('.html'):
                os.startfile(start.filename(self.staging))
        else:
            os.startfile(self.staging)
        storage.abort(self)
















class CitySync(CityKid):

    usage = """Usage: citysync filename.cty
"""

    bash = "c:/cygwin/bin/bash.exe"

    rsync_target = binding.Delegate('vars')
    
    def _run(self):
        try:
            fname, = self.parsed_args
        except ValueError:
            return self._invocationError("Only a CityDesk filename is needed")

        staging = os.path.realpath(self.staging)
        drive, staging = os.path.splitdrive(staging)
        staging = '"/cygdrive/%s%s/"' % (
            drive[:1].lower(), staging.replace(os.sep, os.altsep)
        )
        storage.begin(self)
        cmdline = "%s -lc 'rsync -rzt --stats %s %s'" % (
            self.bash, staging, self.rsync_target
        )
        storage.abort(self)
        print cmdline
        retval = os.system(cmdline)
        raw_input("Done:")













class CityBlog(CityKid):
    blogger_blogid = blogger_userid = blogger_password = \
        binding.Delegate('vars')

    usage = """Usage: cityblog filename.cty itemnumber
"""
    def _run(self):
        try:
            try:
                fname, itemNo = self.parsed_args
            except ValueError:
                return self._invocationError("Two arguments are required")

            storage.begin(self)
            article = self.items.get(self.itemNo)
            if not isinstance(article, data.Article) or article.extra2:
                return self._invocationError(
                    "%s: Not an unpublished article" % article.path
                )

            print "uploading", article.path
            from citykid.blogger import Blog, Post
            p = Post(
                str(article.headline), article.body.encode('utf8'),
                article.modified
            )
            Blog(
                self.blogger_blogid, self.blogger_userid, self.blogger_password
            ).add(p)

            article.extra2 = p.postId
            article.publishAs = p.postURL.split('/')[-1].split('.')[0]
            storage.commit(self)
            raw_input("Done: ")
        except SystemExit:
            raise
        except:
            t,v,tb = sys.exc_info()
            from traceback import print_exception; print_exception(t,v,None)
            import pdb; pdb.post_mortem(tb)

