#!/usr/bin/perl -w
use strict; 

$|++; 

use lib qw(
    ./ 
    ./DADA 
    ./DADA/perllib
    
    ../ 
    ../DADA 
    ../DADA/perllib
    
);




use Getopt::Long;

my $verbose = 0; 


if(!$ENV{GATEWAY_INTERFACE}){ 
    # this (hopefully) means we're running on the cl...

} else { 

    require CGI; 
    my $q = new CGI; 
    print $q->header();
    print '<pre>'; 
    $verbose = 1; 
    
}



GetOptions(
    "verbose"    => \$verbose
);




use DADA::Config; 
use DADA::Mail::MailOut; 
use DADA::App::Guts; 
use DADA::Mail::Send;
use DADA::MailingList::Settings; 

&monitor; 


sub monitor { 

    my @available_lists = DADA::App::Guts::available_lists(); 
    
    foreach my $list(@available_lists) { 
    
        print "\nChecking up on: " . $list . "\n"
            if $verbose; 
    
        my @mailouts  = DADA::Mail::MailOut::current_mailouts($list);  
    
        if(! $mailouts[0]){ 
            print "\t*No current mailouts for $list\n"
                if $verbose;
        }
        foreach my $mailing(@mailouts){ 
        
            print "\n\tmailout: " . $mailing->{id} . "\n"
                if $verbose;
            print "\ttype:    " . $mailing->{type} . "\n"
                if $verbose;
            
            my $mailout = DADA::Mail::MailOut->new({ -List => $list }); 
               $mailout->associate_with_id($mailing->{id}); 
            my $status = $mailout->status(); 
                  
            print "\n\t\tStatus:\n\n"
                if $verbose;
            
            foreach(keys %$status){ 
                if($_ eq 'email_fields'){ 
                    print "\t\t* Subject: " . $status->{$_}->{Subject} . "\n"
                        if $verbose;
                }else { 
                
                    print "\t\t* $_  -  " . $status->{$_} . "\n"
                        if $verbose;    
                }
            }
            
            print "\t\t\n"
                if $verbose;
                     
            if($status->{total_sent_out} < $status->{total_sending_out_num}) { 
            
                 my $ls = DADA::MailingList::Settings->new(-List => $list); 
                 my $li = $ls->get; 
                      
                      
                print "\t\t\tMailing is not finished...\n"
                    if $verbose;
            
                if($status->{should_be_restarted} == 1){ 
                
                    print "\t\t\t\tMailing looks like it should be restarted...\n"
                        if $verbose;    
            
                    if($li->{auto_pickup_dropped_mailings} == 1){ 
            
                        print "\t\t\t\t\tauto pickup is ON - restarting mailing NOW!\n"
                            if $verbose;
                         
                         my $mh = DADA::Mail::Send->new($li); 
                         $mh->restart_bulk_send($mailing->{id}, $mailing->{type}); 
 
                         print "\t\t\t\t\tMailing Restarted Successfully. Exiting...\n"
                            if $verbose;
                         return; 
                   
                    }else { 
                    
                        print "\t\t\t\tauto pickup is OFF - not restarting mailing.\n"
                            if $verbose;
                    }
                }else { 
                        print "\t\t\tmailing appears to be still going...\n"
                            if $verbose;
                }
            
            } else { 
            
                print "\t\tMailing is finished.\n"
                    if $verbose;
            
            }
       }
   }
}



=head1 NAME

auto_pickup.pl - lil' app to monitor mailings and auto reload them if they seem to have fallen off the wayside. 


=head1 VERSION

Refer to the version of Dada Mail you're using - NEVER use a version of this proggy with an earlier or later version of Dada Mail. 

=head1 USAGE

This little program can be used two ways: 

=over

=item * Use it like a cgi script. 

Put it in your, B<extensions> directory (ala: cgi-bin/dada/extensions), chmod 755, visit it in your web browser. 

=item * Use it like a command line script 

Change into the directory it's located in, and type: 

 ./auto_pickup.pl

This script takes B<one> argument: B<--verbose>. Tack that flag on: 

 ./auto_pickup.pl --verbose

and you'll receive a nice printout. Call it without any arguments and it won't return anything - it'll just run, and exit. Good for a cronjob.

If you B<do> run it as a cronjob (sort of its intention), make sure to explictily put the absolute paths to the DADA and DADA/perllib directories, or this thing won't run very well. 

=back

=head1 REQUIRED ARGUMENTS

There are no required arguments.  

=head1 OPTIONS

=over

=item --verbose

As explained, prints out what's goin' on. Probably a good option to use, if you've never used this proggy. 

=back

=head1 DESCRIPTION

.... 

=head1 DIAGNOSTICS

None, really.

=head1 CONFIGURATION AND ENVIRONMENT

Out of the box, this script should run as a CGI script, if you place it in the I<cgi-bin/dada/extensions> directory. 

If you want to run it via a cronjob (which is what it's good for, really), follow the below directions: 

To have this script run successfully as a cronjob, you will have to 
change the Path to Dada Mail's Perl Libraries. 

B<NOTE> This is not the same as your path to Perl (which is usually #!/usr/bin/perl). I get this asked frequently. 

You will need to explicitly state where both your path to the regular Perl libs
are, and the Dada Mail libraries are. For example: 

 use lib qw(
 /home/myaccount/www/cgi-bin/dada
 /home/myaccount/www/cgi-bin/dada/DADA
 /home/myaccount/www/cgi-bin/dada/DADA/perllib
 
 
 /usr/local/lib/perl5/site_perl/5.8.0/mach
 /usr/local/lib/perl5/site_perl/5.8.0
 /usr/local/lib/perl5/site_perl
 /usr/local/lib/perl5/5.8.0/BSDPAN
 /usr/local/lib/perl5/5.8.0/mach
 /usr/local/lib/perl5/5.8.0
 ); 

If you don't know where your Perl library is, trying running this via
the command line: 

 perl -e 'print $_ ."\n" foreach @INC'; 

If you do not know how to run the above command, visit Dada Mail in a web browser, log into your list and on the left hand menu and: click, B<About Dada Mail> 

Under B<Script Information>, click the, B<More...> link and under the, B<Perl Library Locations>, select each point that begins with a, "/" and use those as your site-wide path to your perl libraries. 

=head1 DEPENDENCIES

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

Please, let me know if you find any bugs.

=head1 AUTHOR

Justin Simoni 

See: http://mojo.skazat.com/contact

=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006 Justin Simoni All rights reserved. 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA  02111-1307, USA.

=cut
