"""Command-line Client Base Classes"""

import sys
from optparse import OptionParser
from freeswytch import Switch
from peak.events import trellis



































class ClientBase(trellis.Component):
    """Command-line FreeSWITCH client app"""

    host = '127.0.0.1'
    port = 8021
    auth = 'ClueCon'
    usage = "%prog [options]"
    switch_type = Switch
    switch = trellis.value(None)
    exitcode = 0

    def __init__(self, argv=None):
        if argv is None:
            argv = sys.argv[:]
        self.argv = argv

    def build_parser(self):
        parser = OptionParser()
        parser.add_option("-H", "--host", default=self.host,
                  help="connect to HOST",)
        parser.add_option("-p", "--port", type="int", default=self.port,
                  help="connect to PORT",)
        parser.add_option("-a", "--auth", default=self.auth,
                  help="use PASSWORD for authentication", metavar="PASSWORD")
        return parser
                  
    def start(self, args=(), **kw):
        if args:
            self.parser.error("Extra command line arguments: "+str(args))
        self.switch_type.connect(
            host=self.host, port=self.port, credential=self.auth, **kw
        ).addCallbacks(lambda v: setattr(self, 'switch', v), self.fail)
        from twisted.internet import reactor
        reactor.run()
        return self.exitcode        

    def __class_call__(cls, *args, **kw):
        return super(ClientBase, cls).__class_call__(*args, **kw).run()



    def run(self):
        self.parser = self.build_parser()
        return self.start(self.parser.parse_args(self.argv[1:], self)[1])

    def fail(self, reason):
        print >>sys.stderr, reason
        self.stop(1)

    def stop(self, exitcode=0):
        from twisted.internet import reactor
        if reactor.running:
            reactor.stop()
        self.exitcode = exitcode

    @trellis.observer
    def on_close(self):
        if self.switch and self.switch.closed is not False:
            self.fail(self.switch.closed)























class ConferenceClient(ClientBase):
    """Command-line client that works with a specific (named) conference"""

    usage = "%prog [options] conferencename"

    conference_name = None

    @trellis.rule
    def conference(self):
        if self.switch is not None:
            from freeswytch import Conference
            return Conference(self.switch, self.conference_name)

    def start(self, args):
        if not args:
            self.parser.error("Must specify a conference name")
        self.conference_name = args[0]
        return super(ConferenceClient, self).start(args[1:])
        

class DumpClient(ClientBase):
    """Example client: dump all events to stdout"""

    def start(self, args):
        return super(DumpClient, self).start(
            args, watch_events=trellis.Set(['ALL'])
        )

    @trellis.observer
    def print_events(self):
        if self.switch is not None:
            for event in self.switch.events:
                print event
                sys.stdout.flush()
    






