from twisted.internet import defer, protocol
from twisted.python import failure
from peak.events import trellis

class SwitchProtocol(trellis.Component, protocol.Protocol):
    """I bridge a Switch to a connected transport"""

    switch = None

    def dataReceived(self, data):
        self.switch.receivedBytes(data)

    @trellis.observer
    def sendCommands(self):
        for data in self.switch.transmit:
            self.transport.write(data)

    def connectionLost(self, reason=protocol.connectionDone):
        self.switch.close(reason)

class SwitchFactory(protocol.ClientFactory):
    """I help Twisted connect a transport to a Switch"""

    def __init__(self, cls, kw, callback):
        self.cls = cls
        self.kw = kw
        self.callback = callback

    def protocol(self):
        switch = self.cls(**self.kw)
        from twisted.internet import reactor
        reactor.callLater(0, self.callback, switch)
        return SwitchProtocol(switch = switch)

    def clientConnectionFailed(self, connector, reason):
        from twisted.internet import reactor
        if not isinstance(reason, failure.Failure):
            reason = failure.Failure(reason)
        reactor.callLater(0, self.callback, reason)

